@@ -44,6 +44,8 @@ import shutil
 import os
 import pysvn
 import filecmp
+import subprocess
+import time
 
 ## This case simply imports an empty directory into the toplevel of an
 ## empty repository. But, it provides the framework for other classes
@@ -59,7 +61,10 @@ class BaseLoadCase(unittest.TestCase):
         self.url = 'file://' + self.repo
         ## idir is the relative path in repo where things get imported
         self.idir = ''
+        self.username = None
+        self.password = None
         self.extra_args = []
+        self.extra_svn_args = []
 
     def importDirSetUp(self):
         self.importdir = os.path.join(self.tempdir, 'importdir')
@@ -73,26 +78,29 @@ class BaseLoadCase(unittest.TestCase):
         shutil.rmtree(self.tempdir)
 
     def createRepository(self):
-        svnadmin = '/usr/bin/svnadmin'
         repo = os.path.join(self.tempdir, 'repo')
         os.mkdir(repo)
-        os.spawnl(os.P_WAIT, svnadmin, svnadmin, 'create', repo)
+        subprocess.call(['/usr/bin/svnadmin', 'create', repo])
         return repo
 
     def exportRepository(self):
-        svn = '/usr/bin/svn'
-        ret = os.spawnl(os.P_WAIT, svn, svn, 'export', '--quiet', self.url,
-                        os.path.join(self.exportparent, 'export'))
+        args = [ '/usr/bin/svn', 'export', '--quiet' ]
+        args = args + self.extra_svn_args
+        args = args + [self.url, os.path.join(self.exportparent, 'export')]
+
+        ret = subprocess.call(args)
+
         if ret == 0:
             return True
         else:
             return False
 
     def loaddir(self, ldir):
-        svnloaddirs = "./svn-load"
-        ret = os.spawnl(os.P_WAIT, svnloaddirs, svnloaddirs,
-                        self.url, self.idir, ldir, "--no-prompt",
-                        *self.extra_args)
+        args = ['./svn-load', self.url, self.idir, ldir, '--no-prompt']
+        args = args + self.extra_args
+
+        ret = subprocess.call(args)
+
         if ret == 0:
             return True
         else:
@@ -139,6 +147,11 @@ class BaseLoadCase(unittest.TestCase):
             
     def testLoad(self):
         client = pysvn.Client()
+        if self.username:
+            client.set_default_username(self.username)
+        if self.password:
+            client.set_default_password(self.password)
+        
         client.import_(self.importdir, self.url, 'Initial import')
         for d in self.loaddirs:
             self.failUnless(self.loaddir(d))
@@ -183,6 +196,20 @@ class SingleFileContentChange(BaseLoadCase):
         f.write("baz\n")
         f.close()
 
+### We'd need to be able to control the user's svn config file to test this.
+#class AutoProps(BaseLoadCase):
+#    def loadDirsSetUp(self):
+#        BaseLoadCase.loadDirsSetUp(self)
+#        f = open(os.path.join(self.loaddirs[0], 'foo.txt'), 'w')
+#        f.write("baz\n")
+#        f.close()
+#
+#    def testLoad(self):
+#        BaseLoadCase.testLoad(self)
+#        client = pysvn.Client()
+#        proplist = client.proplist('%s/foo.txt' % (self.url))
+#        self.failUnless(<some known property is set>)
+
 class BaseMoveCase(BaseLoadCase):
     # files_old: list of files in "old" side
     # files_new: list of files in "new" side, must be identical in length to
@@ -484,6 +511,60 @@ class GlobHandling(BaseLoadCase):
         self.failUnless(self.exportRepository())
         self.failUnless(self.compare_trees(self.loaddirs[-1], os.path.join(self.exportparent, 'export')))
 
+class TempConfigFile:
+    def __init__(self):
+        self.f = tempfile.NamedTemporaryFile(delete=False)
+        self.path = self.f.name
+
+    def __del__(self):
+        os.unlink(self.path)
+
+class SvnServePasswdFile(TempConfigFile):
+    def __init__(self, userDict):
+        TempConfigFile.__init__(self)
+        self.f.write('[users]\n')
+        users = userDict.keys()
+        for u in users:
+            self.f.write('%s = %s\n' % (u, userDict[u]))
+        self.f.close()
+
+class SvnServeConf(TempConfigFile):
+    def __init__(self, passwd):
+        TempConfigFile.__init__(self)
+        self.f.write('[general]\n')
+        self.f.write('anon-access = none\n')
+        self.f.write('auth-access = write\n')
+        self.f.write('password-db = %s\n' % (passwd))
+        self.f.close()
+
+class AuthenticatedServer(BaseLoadCase):
+    def startServer(self):
+        self.passwd = SvnServePasswdFile({ self.username: self.password })
+        self.conf = SvnServeConf(self.passwd.path)
+        self.port = 8686
+        self.server = subprocess.Popen(['svnserve', '--daemon', '--foreground',
+                                        '--root=%s' % (self.tempdir),
+                                        '--listen-host=127.0.0.1',
+                                        '--listen-port=%d' % (self.port),
+                                        '--config-file=%s' % (self.conf.path)])
+        # Give server time to listen()
+        time.sleep(1)
+
+    def setUp(self):
+        BaseLoadCase.setUp(self)
+        self.username = 'alice'
+        self.password = 'secret'
+        self.extra_args.extend(['--svn_username=%s' % (self.username)])
+        self.extra_args.extend(['--svn_password=%s' % (self.password)])
+        self.extra_svn_args.extend(['--username', self.username])
+        self.extra_svn_args.extend(['--password', self.password])
+        self.startServer()
+        self.url = 'svn://127.0.0.1:%s/repo' % (self.port)
+
+    def tearDown(self):
+        self.server.kill()
+        BaseLoadCase.tearDown(self)
+
 if __name__ == '__main__':
     try:
         defaultTest = sys.argv[1]
